@@ -7,7 +7,6 @@ from PIL import Image
 from time import perf_counter
 from tqdm import tqdm
 from multiprocessing import Pool
-from dataclasses import dataclass
 from transformers import (
     SegformerForSemanticSegmentation,
     SegformerImageProcessor,
@@ -20,7 +19,7 @@ from shapely.ops import unary_union
 
 from ...image_utils import label_mask_to_polygons
 from ...gpd_utils import rdp_by_fraction_of_max_dimension, make_multi_polygons
-from ..ml.inference_results import InferenceResult
+from ..inference_results import InferenceResult
 
 stain_color_map = htk.preprocessing.color_deconvolution.stain_color_map
 
@@ -39,15 +38,6 @@ STANDARD_MM_PER_PX = 0.0002519
 STANDARD_MAG = 40
 
 
-@dataclass
-class SegFormerSSInferenceResult(InferenceResult):
-    """Result class for SegFormer semantic segmentation inference."""
-
-    gdf: gpd.GeoDataFrame
-    mag: float
-    mm_px: float
-
-
 def inference(
     model: str | torch.nn.Module,
     wsi_fp: str,
@@ -66,7 +56,7 @@ def inference(
     nproc: int = 20,
     interior_max_area: int = 100000,
     hematoxylin_channel: bool = False,
-) -> SegFormerSSInferenceResult:
+) -> InferenceResult:
     """Inference using SegFormer semantic segmentation model on a WSI.
 
     Args:
@@ -179,7 +169,7 @@ def inference(
     # Track all predicted polygons.
     wsi_polygons = []
 
-    output = SegFormerSSInferenceResult()
+    results = InferenceResult()
 
     start_time = perf_counter()
 
@@ -251,7 +241,7 @@ def inference(
         print(f"\r    Processed batch {batch_n}.    ", end="")
     print()
 
-    output.add_time("inference", perf_counter() - start_time)
+    results.add_time("inference", perf_counter() - start_time)
 
     # Convert polygons and labels to a GeoDataFrame.
     gdf = gpd.GeoDataFrame(wsi_polygons, columns=["geometry", "label"])
@@ -260,12 +250,12 @@ def inference(
     # touch, this allows merging adjacent tile polygons when dissolving.
     start_time = perf_counter()
     gdf["geometry"] = gdf["geometry"].buffer(buffer)
-    output.add_time("buffer", perf_counter() - start_time)
+    results.add_time("buffer", perf_counter() - start_time)
 
     start_time = perf_counter()
     gdf = gdf.dissolve(by="label", as_index=False)
     gdf = gdf.explode(index_parts=False).reset_index(drop=True)
-    output.add_time("dissolve", perf_counter() - start_time)
+    results.add_time("dissolve", perf_counter() - start_time)
 
     # Scale the geometries.
     gdf["geometry"] = gdf["geometry"].apply(
@@ -284,13 +274,13 @@ def inference(
     gdf = cleanup_pipe.cleanup()
 
     for section_name, time in cleanup_pipe.time.items():
-        output.add_time(section_name, time)
+        results.add_time(section_name, time)
 
-    output.gdf = gdf
-    output.mag = mag
-    output.mm_px = mm_px
+    results.add_field("gdf", gdf)
+    results.add_field("mag", mag)
+    results.add_field("mm_px", mm_px)
 
-    return output
+    return results
 
 
 class SegFormerSSInferenceCleanup:
@@ -330,7 +320,7 @@ class SegFormerSSInferenceCleanup:
                 raise ValueError(f"Label {label} not in label_ranks.")
             gdf.loc[i, "rank"] = label_ranks.index(r["label"])
 
-        self.__version__ = "1.0.1"
+        self.__version__ = "1.0.2"
         self.input_gdf = gdf
         self.small_hole_thr = small_hole_thr
         self.output_gdf = None
@@ -484,17 +474,28 @@ class SegFormerSSInferenceCleanup:
         # Fill the gaps between polygons created by RDP.
         gdf["area"] = gdf["geometry"].area
 
-        gdf_union = gdf["geometry"].union_all()
+        union_geom = gdf["geometry"].union_all()
 
-        # Collect all the holes / interiors.
         interiors = []
 
-        for geom in gdf_union.geoms:
-            for interior in geom.interiors:
+        if union_geom.geom_type == "MultiPolygon":
+            # Collect all the holes / interiors.
+            for geom in union_geom.geoms:
+                for interior in geom.interiors:
+                    interior = Polygon(interior)
+
+                    if interior.area < interior_max_area:
+                        interiors.append(interior)
+        elif union_geom.geom_type == "Polygon":
+            for interior in union_geom.interiors:
                 interior = Polygon(interior)
 
                 if interior.area < interior_max_area:
                     interiors.append(interior)
+        else:
+            raise ValueError(
+                f"Unexpected geometry type: {union_geom.geom_type}"
+            )
 
         # Loop through each hole that was small.
         for interior in tqdm(interiors, desc="Filling holes between polygons"):
